package de.XPortLib;

import java.util.Calendar;

import de.XPortLib.Exceptions.DataTransmissionException;
import de.XPortLib.Exceptions.TCPCommunicationException;
import de.XPortLib.Interfaces.XPortControllerIF;




public class XPortController implements XPortControllerIF {
	
	/** Number of I/O pins available on WiPort */
	public static final int NUMCPINS = 3;

	/** Number of Relais on B-Module */
	public static final int NUMRELAIS = 7;

	/** instance of a Communication class object */
	private Communication XPort;
	
	/** the remote IP address of the XPort */
	protected String mIpAddress;
	
	/** the remote port of the XPort */
	protected int mPort;
	
	/** default constructor for initialization */
	public XPortController() {
		XPort = new Communication();
	}
	
	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#getIpAddres()
	 */
	public synchronized String getIpAddress() {
		return mIpAddress;
	}

	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#setIpAddres(java.lang.String)
	 */
	public synchronized void setIpAddress(String ipAddres) {
		mIpAddress = ipAddres;
	}

	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#getPort()
	 */
	public synchronized int getPort() {
		return mPort;
	}

	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#setPort(int)
	 */
	public synchronized void setPort(int port) {
		mPort = port;
	}
	
	
	/**
	 * method to execute a command and receive the answer.
	 * Does also perform connect / reconnect
	 * @param command
	 * 			Command object to be executed
	 * @throws Exception
	 * @throws IOException
	 */
	private void executeCommand(Command command) throws TCPCommunicationException, DataTransmissionException {
	
		try {
			/* Connect to XPort and send command */
			XPort.connect(mIpAddress, mPort);

			boolean success = false;
			int retries = 1;
			
			//retry to transmit several times due to random failures
			do {
				try {
					transmitCommand(command);
					receiveAnswer(command);
					success = true;
				} catch (DataTransmissionException ex) {
		    		if (retries++ > 4) throw ex;
				}
			} while (!success);
		}
		catch (TCPCommunicationException ex) {
			throw ex;
		}
		catch (DataTransmissionException ex) {
			throw ex;
		}
		finally {
			try {
				XPort.disconnect();
			} catch (Exception ex) {}
//			try {
//				Thread.sleep(50);
//			} catch (InterruptedException e) { }
		}
	}
	

	/**
	 * method to execute a command and receive the answer.
	 * Does NOT perform connect / reconnect
	 * @param command
	 * 			Command object to be executed
	 * @throws Exception
	 * @throws IOException
	 */
	private void executeCommandConnected(Command command) throws TCPCommunicationException, DataTransmissionException {
		boolean success = false;
		int retries = 1;
		
		//retry to transmit several times due to random failures
		do {
			try {
				transmitCommand(command);
				receiveAnswer(command);
				success = true;
			} catch (DataTransmissionException ex) {
	    		if (retries++ > 4) throw ex;
			}
		} while (!success);
	}

	
	/**
	 * executes a command object
	 * @param command
	 * 			Command object to execute
	 * @throws Exception
	 * @throws IOException
	 */
	private void transmitCommand(Command command) throws TCPCommunicationException {
		XPort.send(command.getFullCommand());

		/* Do we have to receive data before receiving the answer frame? */
		if (command.getPayload() != null) {
			try {
				Thread.sleep(20);
			} catch (InterruptedException e) {}
			XPort.send(command.getPayload());
		}

	}		
	
	
	/**
	 * sleep procedure to wait for data
	 * @throws Exception
	 */
	private void waitForData() {
		int timeout = 0;
		while ((!XPort.DataReady()) & (timeout < 20)) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException ex) {
			}
			timeout++;
		}		
	}
	
	
	/**
	 * receives the answer from XPort
	 * @param command
	 * 			the Command object that was executed before
	 * @throws Exception
	 * @throws IOException
	 */
	private void receiveAnswer(Command command) throws DataTransmissionException, TCPCommunicationException {
		int length;
		
		/* Do we have to receive data before receiving the answer frame? */

		if (command.getExpectedRetourSize() > 0) {
			int[] retour = new int[command.getExpectedRetourSize()];
			
			// Wait for retour
			waitForData();
			
			length = XPort.readBuffer(retour, retour.length);
			if (length != retour.length)
				throw (new DataTransmissionException("Not enough Data received"));
			else {
				command.setRetour(retour);
			}
		}
		
		// Wait for answer
		waitForData();
		int[] answer = new int[Command.CMD_LENGTH];

		length = XPort.readBuffer(answer, Command.CMD_LENGTH);


		if ((answer[0] != Command.START_CODE) | (answer[1] != command.getCommand())
				| (answer[Command.CMD_LENGTH - 1] != Command.STOP_CODE)
				| (length != Command.CMD_LENGTH)) {
			throw (new DataTransmissionException("No correct answer from B-Module"));
		}
		else {
			command.setAnswer(answer);
		}
	}
		
	
	/**
	 * converts byte status to boolean array
	 * @param status
	 * @return
	 */
	private boolean[] convertStatusToArray(int status) {
		boolean[] statusArray = new boolean[NUMRELAIS];
		for (int i = 0; i < NUMRELAIS; i++) {
			if ((status & (1 << i)) > 0)
				statusArray[i] = true;
			else
				statusArray[i] = false;
		}
		return statusArray;
	}
	
	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#readRelais()
	 */
	public synchronized boolean[] readRelais() throws DataTransmissionException, TCPCommunicationException {
		return convertStatusToArray(readRelaisInt());
	}
	
	/**
	 * reads the current relais status
	 * @return
	 * 			boolean array with status for each relais
	 * @throws Exception
	 * @throws IOException
	 */
	private int readRelaisInt() throws DataTransmissionException, TCPCommunicationException {
		Command readRelais = new Command(Command.CMD_READRELAIS);
		executeCommand(readRelais);	
		return readRelais.getAnswer()[2];
	}
	
	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#switchRelais(int)
	 */
	public synchronized boolean[] toggleRelais(int channel) throws DataTransmissionException, TCPCommunicationException {
		
		// First ask for State of Relais
    	int initalStatus = readRelaisInt();
    	int finalStatus = initalStatus;
    	int[] data = { initalStatus };

		// alle anderen Relais lassen, wie sie waren
		data[0] &= (~(1 << channel));

		// den einen Kanal Invertieren und dann dazupacken
		data[0] += (~initalStatus) & (1 << channel);

		Command switchRelais = new Command(Command.CMD_SETRELAIS, data);
		
		//switch while initial status equals final status -> no change yet
		do {
			executeCommand(switchRelais);
			finalStatus = switchRelais.getAnswer()[2];
		} while (finalStatus == initalStatus);
		return convertStatusToArray(finalStatus);
	}
	
	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#readTimer(int)
	 */
	public synchronized TimerEvent readTimer(int nr) throws DataTransmissionException, TCPCommunicationException {
		int[] data = { nr };
		Command readTimer = new Command(Command.CMD_GETEVENT, data, TimerEvent.SIZE);
		executeCommand(readTimer);
		TimerEvent timer = new TimerEvent(readTimer.getRetour(), readTimer.getAnswer()[2]);
		timer.setTimerNr(nr);
		return timer;
	}
	
	
	/**
	 * reads the timerevent at given position
	 * does NOT perform connect / disconnect
	 * @param nr
	 * 			the position
	 * @return
	 * 			the TimerEvent object retrieved from XPort
	 * @throws Exception
	 * @throws IOException
	 */
	private TimerEvent readTimerConnected(int nr) throws DataTransmissionException, TCPCommunicationException {
		int[] data = { nr };
		Command readTimer = new Command(Command.CMD_GETEVENT, data, TimerEvent.SIZE);
		executeCommandConnected(readTimer);
		TimerEvent timer = new TimerEvent(readTimer.getRetour(), readTimer.getAnswer()[2]);
		timer.setTimerNr(nr);
		return timer;
	}
	
	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#readTimers()
	 */
	public synchronized TimerEvent[] readTimers() throws DataTransmissionException, TCPCommunicationException {
		try {
			/* Connect to XPort and send command */
			XPort.connect(mIpAddress, mPort);

			
			int i = 1;
			//read first timer to receive total amount of timers
			TimerEvent event = readTimerConnected(0);
			int totalTimers = event.getTotalTimers();
			TimerEvent[] timers = new TimerEvent[totalTimers];
			timers[0] = event;
			
			//read remaining timers
			while(i < totalTimers) {
				timers[i]=readTimerConnected(i++);
			}
			return timers;
		}
		catch (TCPCommunicationException ex) {
			throw ex;
		}
		catch (DataTransmissionException ex) {
			throw ex;
		}
		finally {
			try {
				XPort.disconnect();
			}
			catch (Exception ex) {}
		}
		
	}
	
	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#writeTimer(int, de.XPortLib.TimerEvent)
	 */
	public synchronized int writeTimer(int nr, TimerEvent timer) throws DataTransmissionException, TCPCommunicationException {
		int[] data = { nr };
		Command writeTimer = new Command(Command.CMD_SETEVENT, data, 0, timer.getSerial());
		executeCommand(writeTimer);
		return writeTimer.getAnswer()[2];
	}		
	
	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#deleteTimer(int)
	 */
	public synchronized int deleteTimer(int nr) throws DataTransmissionException, TCPCommunicationException {
		int[] data = { nr };
		Command writeTimer = new Command(Command.CMD_DELETEEVENT, data);
		executeCommand(writeTimer);
		return writeTimer.getAnswer()[2];
	}	

	
	/* (non-Javadoc)
	 * @see de.XPortLib.XPortControllerIF#setDateTime(java.util.Calendar)
	 */
	public synchronized void setDateTime(Calendar cal) throws DataTransmissionException, TCPCommunicationException {
		int[] date = { cal.get(Calendar.DAY_OF_MONTH),
			(cal.get(Calendar.MONTH) + 1),
			(cal.get(Calendar.YEAR) >> 8),
			(cal.get(Calendar.YEAR) & 255),
			cal.get(Calendar.DAY_OF_WEEK) };

		int[] time = { cal.get(Calendar.HOUR_OF_DAY),
			cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND) };

		Command setDate = new Command(Command.CMD_SETDATE, date);
		Command setTime = new Command(Command.CMD_SETTIME, time);
		
		executeCommand(setDate);
		executeCommand(setTime);		
	}
}
